/* libaudit.h -- 
 * Copyright 2004, 2005 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Authors:
 *	Steve Grubb <sgrubb@redhat.com>
 *	Rickard E. (Rik) Faith <faith@redhat.com>
 */
#ifndef _LIBAUDIT_H_
#define _LIBAUDIT_H_

#include <asm/types.h>
#include <stdint.h>
#include <sys/socket.h>
#include <linux/netlink.h>
#include <linux/audit.h>
#include <stdarg.h>
#include <syslog.h>
#include <linux/fs.h>	/* For watch */

#define AUDIT_FIRST_USER_MSG    1100    /* First user space message */
#define AUDIT_LAST_USER_MSG     1199    /* Last user space message */
#define AUDIT_USER_AUTH         1100    /* User space authentication */
#define AUDIT_USER_ACCT         1101    /* User space acct change */
#define AUDIT_USER_MGMT         1102    /* User space acct management */
#define AUDIT_CRED_ACQ          1103    /* User space credential acquired */
#define AUDIT_CRED_DISP         1104    /* User space credential disposed */
#define AUDIT_USER_START        1105    /* User space session start */
#define AUDIT_USER_END          1106    /* User space session end */
#define AUDIT_USER_AVC          1107    /* User space avc message */
#define AUDIT_USER_CHAUTHTOK	1108	/* User space acct attr changed */
#define AUDIT_USER_ERR		1109	/* User space acct state err */
#define AUDIT_CRED_REFR         1110    /* User space credential refreshed */
#define AUDIT_USYS_CONFIG       1111    /* User space system config change */

#define AUDIT_FIRST_DAEMON	1200
#define AUDIT_LAST_DAEMON	1299
#define AUDIT_DAEMON_RECONFIG	1204	/* Audit daemon should reconfigure */

#define AUDIT_FIRST_EVENT	1300
#define AUDIT_LAST_EVENT	1399
#define AUDIT_CWD		1307	/* Current working directory */
#define AUDIT_FS_INODE		1308	/* File system inode */

#define AUDIT_FIRST_SELINUX	1400
#define AUDIT_LAST_SELINUX	1499

/* This is new list defines from audit.h */
#ifndef AUDIT_FILTER_USER
#define AUDIT_FILTER_USER       0x00    /* Apply rule to user-generated messages */
#endif
#ifndef AUDIT_FILTER_TASK
#define AUDIT_FILTER_TASK       0x01    /* Apply rule at task creation (not syscall) */
#endif
#ifndef AUDIT_FILTER_ENTRY
#define AUDIT_FILTER_ENTRY      0x02    /* Apply rule at syscall entry */
#endif
#ifndef AUDIT_FILTER_WATCH
#define AUDIT_FILTER_WATCH      0x03    /* Apply rule to file system watches */
#endif
#ifndef AUDIT_FILTER_EXIT
#define AUDIT_FILTER_EXIT       0x04    /* Apply rule at syscall exit */
#endif
#define AUDIT_FILTER_MASK	0x07	/* Mask to get actual filter */
#ifndef AUDIT_FILTER_PREPEND
#define AUDIT_FILTER_PREPEND    0x10    /* Prepend to front of list */
#endif
#define AUDIT_FILTER_UNSET	0x80	/* This value means filter is unset */

/* This is the machine type list */
typedef enum {
	MACH_X86=0,
	MACH_86_64,
	MACH_IA64,
	MACH_PPC64,
	MACH_PPC,
	MACH_S390X,
	MACH_S390
} machine_t;

#ifndef AUDIT_FILTERKEY_MAX
/* From the watch patch  - 32 byte max key size */
#define AUDIT_FILTERKEY_MAX    32

/* Structure to transport watch data to and from the kernel */
struct watch_transport {
	uint32_t dev_major;
	uint32_t dev_minor;
	uint32_t perms;
	uint32_t valid;
	uint32_t pathlen;
	uint32_t fklen;
	char buf[0];
};
#endif

struct audit_watch {
	uint32_t dev_major;
	uint32_t dev_minor;
	uint32_t perms;
	uint32_t valid;
	uint32_t namelen;
	uint32_t fklen;
	char     *name;
	char     *filterkey;
};

/* data structure for who signaled the audit daemon */
struct audit_sig_info {
        uid_t           uid;
        pid_t           pid;
};

/* defines for audit subsystem */
#define MAX_AUDIT_MESSAGE_LENGTH    8460 /* PATH_MAX*2+11+256+1 */
struct audit_message {
	struct nlmsghdr nlh;
	char   data[MAX_AUDIT_MESSAGE_LENGTH];
};

// internal - forward declaration
struct daemon_conf;

struct audit_reply {
	int                      type;
	int                      len;
	struct nlmsghdr         *nlh;
	struct audit_message     msg;

	/* Using a union to compress this structure since only one of
	 * the following should be valid for any packet. */
	union {
	struct audit_status     *status;
	struct audit_rule       *rule;
	struct audit_login      *login;
	const char              *message;
	struct nlmsgerr         *error;
	struct watch_transport  *watch;
	struct audit_sig_info   *signal_info;
	struct daemon_conf      *conf;
	};
};

struct auditd_reply_list {
	struct audit_reply reply;
	struct auditd_reply_list *next;
};

/* Messages */
typedef enum { MSG_STDERR, MSG_SYSLOG, MSG_QUIET } message_t;
typedef enum { DBG_NO, DBG_YES } debug_message_t;
void set_aumessage_mode(message_t mode, debug_message_t debug);

/* General */
typedef enum { GET_REPLY_BLOCKING=0, GET_REPLY_NONBLOCKING } reply_t;
extern int  audit_open(void);
extern void audit_close(int fd);
extern int  audit_get_reply(int fd, struct audit_reply *rep, reply_t block, 
		int peek);
extern uid_t audit_getloginuid(void);
extern int  audit_setloginuid(uid_t uid);
extern int audit_detect_machine(void);

/* Translation functions */
extern int        audit_name_to_field(const char *field);
extern const char *audit_field_to_name(int field);
extern int        audit_name_to_syscall(const char *sc, int machine);
extern const char *audit_syscall_to_name(int sc, int machine);
extern int        audit_name_to_flag(const char *flag);
extern const char *audit_flag_to_name(int flag);
extern int        audit_name_to_action(const char *action);
extern const char *audit_action_to_name(int action);
extern int        audit_name_to_msg_type(const char *msg_type);
extern const char *audit_msg_type_to_name(int msg_type);
extern int        audit_name_to_machine(const char *machine);
extern const char *audit_machine_to_name(int machine);
extern unsigned int audit_machine_to_elf(int machine);
extern int          audit_elf_to_machine(unsigned int elf);

/* AUDIT_GET */
extern int audit_request_status(int fd);
extern int audit_is_enabled(int fd);

/* AUDIT_SET */
typedef enum { WAIT_NO, WAIT_YES } rep_wait_t;
extern int  audit_set_pid(int fd, uint32_t pid, rep_wait_t wmode);
extern int  audit_set_enabled(int fd, uint32_t enabled);
extern int  audit_set_failure(int fd, uint32_t failure);
extern int  audit_set_rate_limit(int fd, uint32_t limit);
extern int  audit_set_backlog_limit(int fd, uint32_t limit);

/* AUDIT_LIST */
extern int  audit_request_rules_list(int fd);
extern int  audit_request_watch_list(int fd);

/* SIGNAL_INFO */
extern int audit_request_signal_info(int fd);

/* AUDIT_WATCH */
extern int audit_insert_watch(int fd, struct audit_watch *req);
extern int audit_remove_watch(int fd, struct audit_watch *req);

/* AUDIT_ADD */
extern int  audit_add_rule(int fd, struct audit_rule *rule,
                           int flags, int action);

/* AUDIT_DEL */
extern int  audit_delete_rule(int fd, struct audit_rule *rule,
                              int flags, int action);

/* AUDIT_USER */
//deprecated - do not use!
extern int audit_vlog(int fd, const char *fmt, va_list ap); //+dbus
extern int audit_send_message_if_enabled(int fd, const char *message); //+pam

// New USER message interfaces
extern int audit_log_if_enabled(int fd, int type, const char *fmt, ...)//passwd
#ifdef __GNUC__
	__attribute__ ((format (printf, 3, 4)));
#else
	;
#endif
extern int audit_log(int fd, int type, const char *fmt, ...);
extern int audit_send_message(int fd, int type, const char *message);
extern int audit_log_avc(int fd, int type, const char *fmt, va_list ap);//dbus
extern int audit_send_user_message(int fd, int type, const char *message);//pam

/* Rule-building helper functions */
extern int  audit_rule_syscall(struct audit_rule *rule, int scall);
extern int  audit_rule_syscallbyname(struct audit_rule *rule,
                                     const char *scall);
extern int  audit_rule_field(struct audit_rule *rule, int field,
                             unsigned int value);
extern int  audit_rule_fieldpair(struct audit_rule *rule, const char *pair, 
		int flags);
extern void audit_rule_free(struct audit_rule *rule);

#endif

